//*******************************************************************************
//                            KERNELCHIP
//  TCP/IP Client class realization
//  Please visit http://www.kernelchip.ru for details
//  Copyrights 2011
//*******************************************************************************

//#include "stdafx.h"
#include <stdio.h>

#include <Winsock2.h>
#include "KeTCP_Client.h"

//*******************************************************************************
bool KeTCP_Client::MakeInitiolization( )
{
  static bool wasWSAStartUp = false;
  if( !wasWSAStartUp )
  {
    WORD wVersionRequested;
    WSADATA wsaData;   
    
    wVersionRequested = MAKEWORD( 2, 0 );    
    if( WSAStartup(wVersionRequested, &wsaData) != 0 ) return false;
                   
    if( LOBYTE(wsaData.wVersion) != 2 || 
        HIBYTE(wsaData.wVersion) != 0 ) 
    {
        return false;
    } 
    wasWSAStartUp = true;
  }    
  return true;
}

//*******************************************************************************
KeTCP_Client::KeTCP_Client( )
{  
  par.remotehost[0] = '\0';
  par.remoteport = 0;
  par.timeout    = 0;
  sockfd = -1;  
}

//*******************************************************************************
KeTCP_Client::SetConnectionParams(char* sHost, unsigned int uiPort )
{
  strcpy( par.remotehost, sHost );
  par.remoteport = uiPort;
}

//*******************************************************************************
KeTCP_Client::~KeTCP_Client()
{
  Disconnect();
}

//*******************************************************************************
bool KeTCP_Client::Reconnect()
{  
  if( !MakeInitiolization() ) return false;

  struct hostent *he;
  struct sockaddr_in host_addr;
  struct sockaddr_in local_addr;
  char connecthost[MAX_PATH]="";
  int  connectport;

  Disconnect();
 
  strcpy(connecthost,par.remotehost);
  connectport = par.remoteport; 
      
  host_addr.sin_family = AF_INET; 
  host_addr.sin_port = htons( connectport );   

  if( !(he=gethostbyname(connecthost)) ) 
  {
    if((host_addr.sin_addr.s_addr = inet_addr( connecthost)) == INADDR_NONE) 
    {      
      return false;
    }
  }
  else
      host_addr.sin_addr = *((struct in_addr *)he->h_addr);           

  memset( &(host_addr.sin_zero), '\0', 8 );
  if( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) == SOCKET_ERROR ) 
  {    
    return false;
  }

  int one = 1;
  if( setsockopt( sockfd, SOL_SOCKET, SO_REUSEADDR, (char *) &one, sizeof( one )) == SOCKET_ERROR )
  {    
    return false;
  }

  if( connect(sockfd, (struct sockaddr *)&host_addr, sizeof(struct sockaddr)) == SOCKET_ERROR )
  {      
    return false;
  }

  connectport = htons( host_addr.sin_port );
  strcpy( connecthost, inet_ntoa(host_addr.sin_addr) );

  int len = sizeof( local_addr );
  if( getsockname(sockfd, (struct sockaddr *)&local_addr, &len) == SOCKET_ERROR )
  {     
    return false;
  }
  
  return true; 
}

//*******************************************************************************
int KeTCP_Client::Recv( char* buff, long szMax )
{       
  struct timeval tv;
  fd_set setForRead;
  fd_set setForWrite;
  fd_set setForError;
  int res;
  int dwRecv; 

  tv.tv_sec = par.timeout > 0 ? par.timeout : 0;
  tv.tv_usec = 0;

  FD_ZERO( &setForRead );
  FD_ZERO( &setForWrite );
  FD_ZERO( &setForError );

  FD_SET( sockfd, &setForRead );
  FD_SET( sockfd, &setForWrite );
  FD_SET( sockfd, &setForError );


  res = select( sockfd+1, &setForRead, NULL, &setForError, ((par.timeout > 0) ? &tv : NULL) );

  if( res == 0 || res == SOCKET_ERROR ) 
  {      
     return false;
  } 
  else
  {
    if( FD_ISSET(sockfd, &setForError) )
    {     
      Disconnect();
      return false;
    }    
    if( FD_ISSET(sockfd, &setForRead) )
    {
      if( (dwRecv = recv(sockfd, buff, szMax, 0)) == SOCKET_ERROR )
      {          
        Disconnect();
        return false;
      }
      else if( dwRecv == 0 )
      {          
        Disconnect();
        return false;
      }        
    }
  }
  return dwRecv;
}

//*******************************************************************************
bool KeTCP_Client::Send( char *buffer, long sz )
{    
  struct timeval tv;
  fd_set setForRead;
  fd_set setForWrite;
  fd_set setForError;
  int res;
  int dwRecv; 
  int i = 0;

  tv.tv_sec = par.timeout > 0 ? par.timeout : 0;
  tv.tv_usec = 0;

  FD_ZERO( &setForRead );
  FD_ZERO( &setForWrite );
  FD_ZERO( &setForError );

  FD_SET( sockfd, &setForRead );
  FD_SET( sockfd, &setForWrite );
  FD_SET( sockfd, &setForError );

  res = select( sockfd+1, NULL, &setForWrite, &setForError, par.timeout > 0 ? &tv : NULL );
  if( res == 0 ) 
  {      
    return false;
  }
  else if( res == SOCKET_ERROR ) 
  {      
    Disconnect();
    return false;
  }
  else
  {
    if( FD_ISSET(sockfd, &setForError) )
    {      
      Disconnect();
      return false;
    }
    
    if( FD_ISSET(sockfd, &setForWrite) )
    {     
      while( sz )
      {
        if( (i = send(sockfd, buffer, (size_t) sz, 0)) != sz )
        {
          if( i == SOCKET_ERROR )
          {              
            return false;
          }
          else if( i )
          {                
            memmove( buffer, buffer+i, (size_t)(sz-i) ); 
            sz -= i;
          }
        }
        else sz = 0;        
      }   
    }
  }
  return true;
}

//*******************************************************************************
bool KeTCP_Client::Disconnect( )
{
  if( sockfd != -1 )
  {
    if( closesocket(sockfd) == SOCKET_ERROR )
    {      
      sockfd = -1;
      return false;
    }  
    sockfd = -1;    
  }  
  return true;
}

//*******************************************************************************
bool KeTCP_Client::IsConnected( )
{
  if( sockfd != -1 ) return true;
  return false;  
}

//*******************************************************************************
//*******************************************************************************
